'''
Created on 2020-09-19

@author: wf
'''
import os.path
import tempfile
import unittest
from pathlib import Path

from lodstorage.storageconfig import StorageConfig

import geograpy
import getpass
from geograpy.locator import Locator, City,CountryManager, Location, LocationContext
from collections import Counter
from lodstorage.uml import UML
import re
from tests.basetest import Geograpy3Test

class TestLocator(Geograpy3Test):
    '''
    test the Locator class from the location module
    '''   
    
    def lookupQuery(self,viewName,whereClause):
        loc=Locator.getInstance()
        queryString=f"SELECT * FROM {viewName} where {whereClause} AND pop is not NULL ORDER by pop desc"
        lookupRecords=loc.sqlDB.query(queryString)
        return lookupRecords
    
    def checkExpected(self,lod,expected):
        emap={}
        found={}
        for key,value in expected:
            emap[key]=value
        for record in lod:
            name=record["name"]
            pop=record["pop"]
            if name in emap and pop> emap[name]:
                found[name]=record
                if self.debug:
                    print(f"{name}:{pop:.0f}")
    
        self.assertEqual(len(found),len(emap))
        
    def testHasViews(self):
        '''
        test that the views are available
        '''
        loc=Locator.getInstance()
        viewsMap=loc.sqlDB.getTableDict(tableType="view")
        for view in ["CityLookup","RegionLookup","CountryLookup"]:
            self.assertTrue(view in viewsMap)
       
    
    def testCityLookup(self):
        '''
        test the cityLookup to city/region/country object cluster 
        '''
        cityLookupRecords=self.lookupQuery("CityLookup", "label in ('Berlin','Paris','Athens','Singapore')")
        expected=[("Berlin",3644000),("Paris",2175000),("Athens",600000),("Singapore",5800000)]
        self.checkExpected(cityLookupRecords,expected)   
         
    def testRegionLookup(self):
        '''
        test region Lookup
        '''   
        regionLookupRecords=self.lookupQuery("RegionLookup", "label in ('CA')")
        expected=[("California",39000000)]
        self.checkExpected(regionLookupRecords,expected) 
        
    def testCountryLookup(self):
        '''
        test country Lookup
        '''
        #self.debug=True
        countryLookupRecords=self.lookupQuery("CountryLookup", "label in ('CA')")
        expected=[("Canada",37000000)]
        self.checkExpected(countryLookupRecords,expected) 
        
    def testIsoRegexp(self):
        '''
        test regular expression for iso codes
        '''
        loc=Locator.getInstance()
        self.assertFalse(loc.isISO('Singapore'))   
         
        query="""
select distinct iso from countries
union 
select distinct iso from regions
"""     
        loc.populate_db()
        isocodeRecords=loc.sqlDB.query(query)
        for isocodeRecord in isocodeRecords:
            isocode=isocodeRecord['iso']
            if isocode:
                isIso=loc.isISO(isocode)
                if not isIso and self.debug:
                    print(isocode)
                self.assertTrue(isIso)
        
    def testWordCount(self):
        '''
        test the word count 
        '''
        loc=Locator.getInstance()
        query="SELECT name from CITIES"
        nameRecords=loc.sqlDB.query(query)
        if self.debug:
            print ("testWordCount: found %d names" % len(nameRecords))
        wc=Counter()
        for nameRecord in nameRecords:
            name=nameRecord['name']
            words=re.split(r"\W+",name)
            wc[len(words)]+=1
        if self.debug:
            print ("most common 20: %s" % wc.most_common(20))
        
    def testUML(self):
        '''
        test adding population data from wikidata to GeoLite2 information
        '''
        Locator.resetInstance()
        loc=Locator.getInstance()  
        loc.populate_db()
        user=getpass.getuser()
        if self.debug:
            print ("current user is %s" % user)
        tableList=loc.sqlDB.getTableList()
        uml=UML()
        title="""geograpy Tables
2021-08-13
[[https://github.com/somnathrakshit/geograpy3 © 2020-2021 geograpy3 project]]"""
        plantUml=uml.tableListToPlantUml(tableList,title=title, packageName="geograpy3")
        showUml=True
        if showUml or self.debug:
            print (plantUml)
            
    def checkExamples(self,examples,countries,debug=False,check=True):
        '''
        
        check that the given example give results in the given countries
        Args:
            examples(list): a list of example location strings
            countries(list): a list of expected country iso codes
        '''
        for index,example in enumerate(examples):
            city=geograpy.locateCity(example,debug=debug)
            if self.debug:
                print("%3d: %22s->%s" % (index,example,city))
            if check:
                self.assertEqual(countries[index],city.country.iso) 
                
    def testGetCountry(self):
        '''
        test getting a country by name or ISO
        '''
        locator=Locator()
        debug=True
        examples=[("DE","Germany"),("US","United States of America"),("USA",None)]
        for name,expectedName in examples:
            country=locator.getCountry(name)
            if debug:
                print(country)
            if expectedName is None:
                self.assertIsNone(country)
            else:
                self.assertIsNotNone(country)
                self.assertEqual(expectedName,country.name)
            
    def testIssue15(self):
        '''
        https://github.com/somnathrakshit/geograpy3/issues/15
        test Issue 15 Disambiguate via population, gdp data
        '''
        examples=['Paris','Vienna', 'Berlin']
        countries=['FR','AT', 'DE']
        self.checkExamples(examples, countries)
        pass
    
    def testIssue17(self):
        '''
        test issue 17:
        
        https://github.com/somnathrakshit/geograpy3/issues/17
        
        [BUG] San Francisco, USA and Auckland, New Zealand should be locatable #17
        '''
        examples=['San Francisco, USA','Auckland, New Zealand']
        countries=['US','NZ']
        self.checkExamples(examples, countries)
        
    def testIssue19(self):
        '''
        test issue 19
        '''
        examples=['Puebla City, Mexico','Newcastle, UK','San Juan, Puerto Rico']
        countries=['MX','GB','US']
        # For Puerto Rico exist two iso codes one as country and one as US region see https://en.wikipedia.org/wiki/Puerto_Rico in the dataset it is recognized as US region
        self.checkExamples(examples, countries)
        
    
        
    def testStackOverflow64379688(self):
        '''
        compare old and new geograpy interface
        '''
        examples=['John Doe 160 Huntington Terrace Newark, New York 07112 United States of America',
                  'John Doe 30 Huntington Terrace Newark, New York 07112 USA',
                  'John Doe 22 Huntington Terrace Newark, New York 07112 US',
                  'Mario Bianchi, Via Nazionale 256, 00148 Roma (RM) Italia',
                  'Mario Bianchi, Via Nazionale 256, 00148 Roma (RM) Italy',
                  'Newark','Rome']
        for example in examples:
            city=geograpy.locateCity(example,debug=False)
            if self.debug:
                print(city)
            
    def testStackOverflow64418919(self):
        '''
        https://stackoverflow.com/questions/64418919/problem-retrieving-region-in-us-with-geograpy3
        '''
        examples=['Seattle']
        for example in examples:
            city=geograpy.locateCity(example,debug=False)
            print(city)
        
    def testProceedingsExample(self):
        '''
        test a proceedings title Example
        '''
        examples=['''Proceedings of the 
IEEE 14th International Conference on 
Semantic Computing, ICSC 2020, 
San Diego, CA, USA, 
February 3-5, 2020''']
        for example in examples:
            places = geograpy.get_place_context(text=example) 
            if self.debug:
                print(places)
            city=geograpy.locateCity(example,debug=False)
            if self.debug:
                print(city)
        
        
    def testDelimiters(self):
        '''
        test the delimiter statistics for names
        '''
        loc=Locator.getInstance()
        loc.populate_db()
        
        ddls=["DROP VIEW IF EXISTS allNames","""CREATE VIEW allNames as select name from countries
        union select name from regions
        union select name from cities"""]
        for ddl in ddls:
            loc.sqlDB.execute(ddl)
        query="SELECT name from allNames"
        nameRecords=loc.sqlDB.query(query)
        show=self.debug
        show=True
        if show:
            print("found %d name records" % len(nameRecords))
        ordC=Counter()
        for nameRecord in nameRecords:
            name=nameRecord["name"]
            for char in name:
                code=ord(char)
                if code<ord("A"):
                    ordC[code]+=1
        for index,countT in enumerate(ordC.most_common(10)):
            code,count=countT
            if show:
                print ("%d: %d %s -> %d" % (index,code,chr(code),count))
    
    
    def testIssue22(self):  
        '''
        https://github.com/somnathrakshit/geograpy3/issues/22
        '''  
        url='https://en.wikipedia.org/wiki/2012_Summer_Olympics_torch_relay'
        places = geograpy.get_geoPlace_context(url = url)
        if self.debug:
            print(places)
        self.assertTrue(len(places.countries)>5)
        self.assertTrue(len(places.regions)>5)
        self.assertTrue(len(places.cities)>20)
            
        
    def testExamples(self):
        '''
        test examples
        '''
        examples=['Paris, US-TX','Amsterdam, Netherlands', 'Vienna, Austria','Vienna, Illinois, US','Paris, Texas', 
                  'Austin, TX','Austin, Texas',
                  ]
        countries=['US','NL','AT','US','US','US','US']
        self.checkExamples(examples, countries,debug=False)

    def testIssue41_CountriesFromErdem(self):
        '''
        test getting Country list from Erdem

        '''
        countryList=CountryManager.fromErdem()
        self.assertEqual(247,len(countryList.countries))
        if self.debug:
            for country in countryList.countries:
                print(country)
                
    def testIssue_42_distance(self):
        '''
        test haversine and location
        '''
        loc1=Location()
        loc1.lat=0
        loc1.lon=0
        loc2=Location()
        loc2.lat=90
        loc2.lon=0
        d=loc1.distance(loc2)
        #self.debug=True
        if self.debug:
            print(d)
        self.assertAlmostEqual(10007.54,d,delta=0.1)

    def testIssue_59_db_download(self):
        '''
        tests the correct downloading of the backup database in different configurations
        '''
        def getConfig(tmpdir:str):
            config=StorageConfig(cacheFile="locations.db", cacheDirName="geograpyTest", cacheRootDir=tmpdir)
            config.cacheFile=f"{config.getCachePath()}/{config.cacheFile}"
            return config

        def downloadAndTestDB(config:StorageConfig, loc:Locator=None, forceUpdate:bool=False):
            '''downloads and tests the downloaded db'''
            if loc is None:
                loc = Locator(storageConfig=config)
            loc.downloadDB(forceUpdate=forceUpdate)
            self.assertTrue(os.path.exists(config.cacheFile))
            self.assertTrue(loc.db_has_data())
            return loc

        # test downloading with no file in dir
        with tempfile.TemporaryDirectory() as tmpdir:
            config=getConfig(tmpdir)
            downloadAndTestDB(config)

        # test downloading with empty file in dir
        with tempfile.TemporaryDirectory() as tmpdir:
            config=getConfig(tmpdir)
            Path(config.cacheFile).touch()   # create empty file
            loc=downloadAndTestDB(config)

            # test downloading with forceUpdate
            # drop a important table to check if it is restored
            loc.sqlDB.execute("DROP TABLE countries")
            self.assertFalse(loc.db_has_data())
            downloadAndTestDB(config,loc=loc, forceUpdate=True)

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



from geograpy.extraction import Extractor
import geograpy
import unittest
from tests.basetest import Geograpy3Test

class TestExtractor(Geograpy3Test):
    '''
    test Extractor
    '''
    
    
    def check(self,places,expectedList):
        '''
        check the places for begin non empty and having at least the expected List of
        elements
        
        Args:
            places(Places): the places to check
            expectedList(list): the list of elements to check
        '''
        if self.debug:
            print(places)
        self.assertTrue(len(places)>0)
        for expected in expectedList:
            self.assertTrue(expected in places)

    def testExtractorFromUrl(self):
        '''
        test the extractor
        '''
        url='https://en.wikipedia.org/wiki/Louvre'
        e = Extractor(url=url)
        e.find_geoEntities()
        self.check(e.places,['Paris','France'])
        
    def testGeograpyIssue32(self):
        '''
        test https://github.com/ushahidi/geograpy/issues/32
        '''
        # do not test since url is unreliable
        return 
        url = "https://www.politico.eu/article/italy-incurable-economy/" 
        places = geograpy.get_geoPlace_context(url = url) 
        if self.debug:
            print(places)
        self.assertSetEqual({'Italy','Germany','France','United States of America','Belgium','Canada'}, set(places.countries))
        self.assertSetEqual({'Rome', 'Brussels', 'Italy','Germany'},set(places.cities))   # Notes: Italy is also city in US-NY, Germany is also city in US-TX
            
    def testGetGeoPlace(self):
        '''
        test geo place handling
        '''
        # 'http://www.bbc.com/news/world-europe-26919928'
        # broken since 2020-10 - returns javascript instead of plain html
        url='https://en.wikipedia.org/wiki/Golden_spike'
        places=geograpy.get_geoPlace_context(url=url)
        debug=self.debug
        #debug=True
        if debug:
            print(places)
        self.assertTrue("Ogden" in places.cities)
        self.assertTrue('Utah' in places.regions)
        self.assertTrue('United States of America' in places.countries)
        
    def testExtractorFromText(self):
        '''
        test different texts for getting geo context information
        '''
        text = """ Perfect just Perfect! It's a perfect storm for Nairobi on a 
        Friday evening! horrible traffic here is your cue to become worse @Ma3Route """
    
        e2 = Extractor(text=text)
        e2.find_entities()
        self.check(e2.places,['Nairobi'])
    
        text3 = """ Risks of Cycling in Nairobi:http://www.globalsiteplans.com/environmental-design/engineering-environmental-design/the-risky-affair-of-cycling-in-nairobi-kenya/ ... via @ConstantCap @KideroEvans @county_nairobi @NrbCity_Traffic """
        e3 = Extractor(text=text3)
        e3.find_entities()
        self.check(e3.places,['Nairobi'])
    
        text4 = """ @DurbanSharks [Africa Renewal]It is early morning in Nairobi, the Kenyan capital. The traffic jam along Ngong """
        e4 = Extractor(text=text4)
        e4.find_entities()
        self.check(e4.places,['Nairobi','Ngong'])
    
        # unicode
        text5 = u""" There is a city called New York in the United States."""
        e5 = Extractor(text=text5)
        e5.find_entities()
        self.check(e5.places,['New York','United States'])
    
        # unicode and two words
        text6 = u""" There is a city called São Paulo in Brazil."""
        e6 = Extractor(text=text6)
        e6.find_entities()
        self.check(e6.places,['São Paulo'])

    def testIssue7(self):
        '''
        test https://github.com/somnathrakshit/geograpy3/issues/7
        disambiguating countries
        '''
        localities=['Vienna, Illinois,','Paris, Texas','Zaragoza, Spain','Vienna, Austria',
                    
                    ]
        expected=[
            {'iso': 'US'},
            {'iso': 'US'},
            {'iso': 'ES'},
            {'iso': 'AT'},
        ]
        for index,locality in enumerate(localities):
            city=geograpy.locateCity(locality,debug=False)
            if self.debug:
                print(f"  {city}")
            self.assertEqual(expected[index]['iso'],city.country.iso)
       
    def testIssue10(self):
        '''
        test https://github.com/somnathrakshit/geograpy3/issues/10
        Add ISO country code
        ''' 
        localities=[
            'Singapore',
            'Beijing, China',
            'Paris, France',
            'Barcelona, Spain',
            'Rome, Italy',
            'San Francisco, US',
            'Bangkok, Thailand',
            'Vienna, Austria',
            'Athens, Greece',
            'Shanghai, China']
        expectedCountry=[
            'SG',
            'CN','FR','ES','IT','US','TH','AT','GR','CN']
        debug=self.debug
        for index,locality in enumerate(localities):
            city=geograpy.locateCity(locality)
            if debug:
                print("  %s" % city)
            self.assertEqual(expectedCountry[index],city.country.iso)
        
    def testIssue9(self):
        '''
        test https://github.com/somnathrakshit/geograpy3/issues/9
        [BUG]AttributeError: 'NoneType' object has no attribute 'name' on "Pristina, Kosovo"
        '''    
        locality="Pristina, Kosovo"
        gp=geograpy.get_geoPlace_context(text=locality)
        if self.debug:
            print("  %s" % gp.countries)
            print("  %s" % gp.regions)
            print("  %s" % gp.cities)
    
    def testStackoverflow62152428(self):
        '''
        see https://stackoverflow.com/questions/62152428/extracting-country-information-from-description-using-geograpy?noredirect=1#comment112899776_62152428
        '''
        examples={2: 'Socialist Republic of Alachua', 3: 'Hérault, France', 4: 'Gwalior, India', 5: 'Zaragoza,España', 6:'Zaragoza, Spain', 7: 'amsterdam ', 8: 'Evesham', 9: 'Rochdale'}  
        for index,text in examples.items():
            places=geograpy.get_geoPlace_context(text=text)
            if self.debug:
                print("example %d: %s" % (index,places.countries))
        
    def testStackoverflow43322567(self):
        '''
        see https://stackoverflow.com/questions/43322567
        '''
        url='https://en.wikipedia.org/wiki/U.S._state'
        e=Extractor(url=url)
        places=e.find_geoEntities()
        self.check(places,['Alabama','Virginia','New York'])
        if self.debug:
            print(places)
        
    def testStackoverflow54712198(self):
        '''
        see https://stackoverflow.com/questions/54712198/not-only-extracting-places-from-a-text-but-also-other-names-in-geograpypython
        '''
        text='''Opposition Leader Mahinda Rajapaksa says that the whole public administration has collapsed due to the constitution council’s arbitrary actions. The Opposition Leader said so in response to a query a journalised raised after a meeting held...'''
        e=Extractor(text)
        places=e.find_geoEntities()
        if self.debug:
            print(places)
        self.assertEqual([],places)
        
        
    def testStackoverflow54077973(self):
        '''
        see https://stackoverflow.com/questions/54077973/geograpy3-library-for-extracting-the-locations-in-the-text-gives-unicodedecodee
        '''
        address = 'Jersey City New Jersey 07306'
        e=Extractor(text=address)
        e.find_entities()
        self.check(e.places,['Jersey','City'])
     

    def testStackOverflow54721435(self):
        '''
        see https://stackoverflow.com/questions/54721435/unable-to-extract-city-names-from-a-text-using-geograpypython
        '''
        text='I live in Kadawatha a suburb of Colombo  Sri Lanka'
        e=Extractor(text=text)
        e.find_entities()
        if self.debug:
            print(e.places)
        
    def testStackoverflow55548116(self):
        '''
        see https://stackoverflow.com/questions/55548116/geograpy3-library-is-not-working-properly-and-give-traceback-error
        '''
        feedContent=['Las Vegas is a city in Nevada']
        placesInFeed=[]
        
        for content in feedContent:
            if content != "":
                e=Extractor(text=content)
                e.find_entities()
                places = e.places
                if self.debug:
                    print(places)
                placesInFeed.append(places)    
        

if __name__ == "__main__":
    unittest.main()



'''
Created on 2021-08-13

@author: wf
'''
import tempfile
import unittest

from lodstorage.storageconfig import StorageConfig

from tests.basetest import Geograpy3Test
from geograpy.locator import LocationContext,LocationManager,CityManager,RegionManager,CountryManager

class TestLocationContext(Geograpy3Test):
    '''
    test the location Context - these are potentially long running tests
    '''
    
    def getStorageConfig(self):
        #config=StorageConfig.getDefault()
        config=LocationContext.getDefaultConfig()
        return config
    
    def checkNoDuplicateWikidataIds(self, locationManager:LocationManager,primaryKey=None,expectedDuplicates=0):
        '''
        check that there are no duplicate Wikidata Q identifiers in the given
        
        '''
        locationsByWikiDataId, duplicates = locationManager.getLookup(primaryKey)
        showLimit=10
        if len(duplicates) > 0:
            for i, duplicate in enumerate(duplicates):
                if i<showLimit:
                    if self.debug:
                        print(f"{i}:{duplicate}")
                else:
                    break
        self.assertTrue(len(duplicates)<= expectedDuplicates)
        return locationsByWikiDataId
    
    def testCountryManager(self):
        '''
        tests the loading and parsing of the RegionManager form the json backup file
        '''
        countryManager = CountryManager(config=self.getStorageConfig())
        countryManager.fromCache()
        self.assertTrue(hasattr(countryManager,'countries'))
        self.assertTrue(len(countryManager.countries) >= 200)
        # check if California is in the list
        countriesByWikidataId=self.checkNoDuplicateWikidataIds(countryManager,"wikidataid")
        self.assertTrue("Q30" in countriesByWikidataId)
        
    def testRegionManager(self):
        '''
        tests the loading and parsing of the RegionManager form the json backup file
        '''
        regionManager = RegionManager(config=self.getStorageConfig())
        regionManager.fromCache()
        self.assertTrue(hasattr(regionManager,'regions'))
        self.assertTrue(len(regionManager.regions) >= 1000)
        regionsByWikidataId = self.checkNoDuplicateWikidataIds(regionManager,"wikidataid",54)
        self.assertTrue("Q99" in regionsByWikidataId)

        
    def testCityManager(self):
        '''
        tests the loading and parsing of the cityList form the json backup file
        '''
        cityManager = CityManager(config=self.getStorageConfig())
        cityManager.fromCache()
        self.assertTrue(hasattr(cityManager, 'cities'))
        self.assertTrue(len(cityManager.cities) >= 200000)
        # check if Los Angeles is in the list (popular city should always be in the list)
        _citiesByWikiDataIdNoDuplicates = self.checkNoDuplicateWikidataIds(cityManager,"wikidataid",304000)   # ToDo: Reduce number of duplicates
        citiesByWikiDataId=cityManager.getLookup("wikidataid", withDuplicates=True)
        self.assertTrue("Q65" in citiesByWikiDataId)
        
    def testLocationContextFromCache(self):
        '''
        test loading LocationContext from cache
        '''
        testCache=False
        if self.inCI() or testCache:
            locationContext = LocationContext.fromCache()
            locationContext.load()
            self.assertTrue(len(locationContext.countries) > 180)
            self.assertTrue(len(locationContext.regions) > 3500)
            self.assertTrue(len(locationContext.cities) > 1000000)


    def testIssue_59_db_download(self):
        '''
        tests if the cache database is downloaded if not present
        '''
        with tempfile.TemporaryDirectory() as tmpdir:
            config=StorageConfig(cacheFile="locations.db", cacheRootDir=tmpdir)
            config.cacheFile = f"{config.getCachePath()}/{config.cacheFile}"
            loc=LocationContext.fromCache(config=config)
            locations=loc.locateLocation("Germany")
            self.assertTrue(len(locations)>0)


if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()


'''
Created on 2021-08-20

@author: wf
'''

from tests.basetest import Geograpy3Test
from geograpy.nominatim import NominatimWrapper
class TestGeopy(Geograpy3Test):
    '''
    test geopy and other nominatim handlers
    '''

    def testNominatim(self):
        '''
        test nominatim results - especially the extra tags
        '''
        examples=[{
            "city":"London",
            "q": "Q84",
            "expected": "England"
        },{
            "city":"Dublin",
            "q": "Q1761",
            "expected": "Éire"
        },{
            "city":"Vienna Austria",
            "q": "Q1741",
            "expected": "Österreich"
        },{
            "city":"Athens, Georgia",
            "q": "Q203263",
            "expected": "Athens-Clarke County"
        },
        # inconsistent results - 2021-12-27
        #{
        #    "city":"St. Petersburg",
        #    "q": "Q656",
        #    "expected": "Санкт-Петербург"
        #},
        {
            # so for St. Petersburg we need to be more specific
            "city":"St. Petersburg, Russia",
            "q": "Q656",
            # to get the russian one
            "expected": "Санкт-Петербург"
        },
        # inconsistent results Q49279759 - 2023-09-29
        #{
        #    "city":"Arlington, VA",
        #    "q": "Q107126",
        #    "expected": "Virginia"
        #}
        {
            "city":"Saint Petersburg, FL",
            "q": "Q49236",
            "expected": "Florida"
        }
        ]
        
        nw=NominatimWrapper()
        show=self.debug
        # show=True
        if show:
            print(nw.cacheDir)
        for example in examples:
            city=example["city"]
            location = nw.geolocator.geocode(city)
            wikidataId=nw.lookupWikiDataId(city)
            q=example["q"]
            expected=example["expected"] 
            if show:
                print(f"{city:<22}:{str(wikidataId):<7}/{str(q):<7}:{location}→{expected}")
            self.assertEqual(str(q),str(wikidataId))
            self.assertTrue(expected in str(location),f"{location}→{expected}")
        pass


'''
Created on 2021-08-19

@author: wf
'''
import unittest
from tests.basetest import Geograpy3Test
from lodstorage.query import QueryManager
from lodstorage.query import Query
from geograpy.locator import LocationContext, Locator
import os
import re

class TestQueries(Geograpy3Test):
    '''
    test queries for documentation, bug reports and the like
    '''

    def getQueryManager(self):
        '''
        get the query manager
        '''
        cachedir=LocationContext.getDefaultConfig().getCachePath()
        scriptDir=os.path.dirname(__file__)
        for path in cachedir,f"{scriptDir}/../geograpy/data":
            qYamlFile=f"{path}/queries.yaml"
            if os.path.isfile(qYamlFile):
                qm=QueryManager(lang='sql',debug=self.debug,queriesPath=qYamlFile)
                return qm
        return None
    
    def documentQueryResult(self,query,lod,tablefmt,show=False):
        '''
        document the query results
        '''
        for record in lod:
            for key in record.keys():
                value=record[key]
                if value is not None:
                    if isinstance(value,str):
                        if re.match(r"Q[0-9]+",value):
                            if tablefmt=="github":
                                record[key]=f"[{value}](https://www.wikidata.org/wiki/{value})"
                            elif tablefmt=="mediawiki":
                                record[key]=f"[https://www.wikidata.org/wiki/{value} {value}]"
        doc=query.documentQueryResult(lod,tablefmt=tablefmt,floatfmt=".0f")
        if show:
            print(doc)
     
    def testQueries(self):
        '''
        test preconfigured queries
        '''
        qm=self.getQueryManager()
        self.assertIsNotNone(qm)
        locator=Locator.getInstance()
        show=self.debug
        #show=True
        for _name,query in qm.queriesByName.items():
            qlod=locator.sqlDB.query(query.query) 
            for tablefmt in ["mediawiki","github"]:
                self.documentQueryResult(query, qlod,tablefmt,show=show)
                
        pass
    
    def testQuery(self):
        '''
        test a single query
        '''
        queries=[("LocationLabel Count","""select count(*),hierarchy 
from location_labels
group by hierarchy"""),
            ("NY example","select * from cityLookup where label='New York City'"),
            ("Berlin example","select * from cityLookup where label='Berlin' order by pop desc,regionName"),
            ("Issue #25","select * from countryLookup where label in ('France', 'Hungary', 'Poland', 'Spain', 'United Kingdom')"),
            ("Issue #25 Bulgaria","select * from cityLookup where label in ('Bulgaria','Croatia','Hungary','Czech Republic') order by pop desc,regionName")]
        for tableName in ["countries","regions","cities"]:
            queries.append((f"unique wikidataids for {tableName}",f"select count(distinct(wikidataid)) as wikidataids from {tableName}"))
            queries.append((f"total #records for {tableName}",f"select count(*) as recordcount from {tableName}"))            
        locator=Locator.getInstance()
        for title,queryString in queries:
            query=Query(name=title,query=queryString,lang="sql")
            qlod=locator.sqlDB.query(queryString) 
            for tablefmt in ["mediawiki","github"]:
                self.documentQueryResult(query, qlod, tablefmt, show=True)

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()


'''
Created on 16.08.2021

@author: wf
'''
import unittest
import tempfile
from lodstorage.storageconfig import StorageConfig
import os
from geograpy.locator import  Locator,LocationContext
from tests.basetest import Geograpy3Test

class TestLocatorDatabase(Geograpy3Test):
    '''
    test the locator database handling
    '''

    
    def testLocatorWithWikiData(self):
        '''
        test Locator 
        '''
        Locator.resetInstance()
        loc=Locator.getInstance()
        #forceUpdate=True
        forceUpdate=False
        loc.populate_db(force=forceUpdate)
        tableList=loc.sqlDB.getTableList()
        expectedCities=800000
        self.assertTrue(loc.db_recordCount(tableList,"countries")>=200)
        self.assertTrue(loc.db_recordCount(tableList,"regions")>=3000)
        self.assertTrue(loc.db_recordCount(tableList,"cities")>=expectedCities)


    def testHasData(self):
        '''
        check has data and populate functionality
        '''
        testDownload=False
        if self.inCI() or testDownload:
            with tempfile.TemporaryDirectory() as cacheRootDir:
                config=StorageConfig(cacheRootDir=cacheRootDir, cacheDirName='geograpy3')
                config.cacheFile = f"{config.getCachePath()}/{LocationContext.db_filename}"
                loc=Locator(storageConfig=config)
                if os.path.isfile(loc.db_file):
                    os.remove(loc.db_file)
                # reinit sqlDB
                loc=Locator(storageConfig=config)
                self.assertFalse(loc.db_has_data())
                loc.populate_db()
                self.assertTrue(loc.db_has_data())


if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()


import geograpy
from geograpy.places import PlaceContext
from geograpy.locator import Locator
import unittest
from tests.basetest import Geograpy3Test

class TestPlaces(Geograpy3Test):
    '''
    test Places 
    '''


    def setUp(self):
        super().setUp(debug=False)
        Locator.resetInstance()
        pass
    
    def testIssue25(self):
        '''
        https://github.com/somnathrakshit/geograpy3/issues/25
        '''
        pc=PlaceContext(place_names=["Bulgaria","Croatia","Czech Republic","Hungary"])
        if self.debug:
            print (pc.countries)

    def testGetRegionNames(self):
        '''
        test getting region names
        '''
        pc=PlaceContext(place_names=["Berlin"])
        regions=pc.getRegions("Germany")
        self.assertEqual(16,len(regions))
        for region in regions:
            if self.debug:
                print(region)
            self.assertTrue(region.iso.startswith("DE"))
        regionNames=pc.get_region_names("Germany")
        self.assertEqual(16,len(regionNames))
        if self.debug:
            print(regionNames)


    def testPlaces(self):
        '''
        test places
        '''
        pc = PlaceContext(['Ngong', 'Nairobi', 'Kenya'],setAll=False)
        pc.setAll()
       
        
        if self.debug:
            print (pc)

        # Ngong is a city in Cameroon and Kenya
        self.assertEqual(2,len(pc.countries))
        self.assertTrue("Kenya" in pc.countries)
        self.assertEqual(2,len(pc.cities))
        cityNames=['Nairobi','Ohio','Amsterdam']
        countries=['Kenya','United States of America','Netherlands']
        for index,cityName in enumerate(cityNames):
            cities=pc.cities_for_name(cityName)
            country=cities[0].country
            self.assertEqual(countries[index],country.name)
    
        pc = PlaceContext(['Mumbai'])
        if self.debug:
            print(pc)
            
    def testIssue49(self):
        '''
        country recognition
        '''
        show=self.debug
        texts=['United Kingdom','UK','Great Britain','GB','United States']
        expected=["United Kingdom","United Kingdom","United Kingdom","United Kingdom","United States of America"]
        if show:
            print("lookup with geograpy.get_geoPlace_context")
        for text in texts:
            countries=geograpy.get_geoPlace_context(text=text).countries
            if show:
                print (f"{text}:{countries}")
        if show:
            print("lookup with PlaceContext")
        for i,text in enumerate(texts):
            pc=PlaceContext([text])  
            pc.set_countries()
            if show:
                print (f"{text}:{pc.countries}")
            self.assertEqual([expected[i]],pc.countries)


if __name__ == "__main__":
    unittest.main()



'''
Created on 2020-09-23

@author: wf
'''
import unittest
from geograpy.wikidata import Wikidata
from geograpy.locator import Country
import getpass
from tests.basetest import Geograpy3Test
from lodstorage.sql import SQLDB
from lodstorage.storageconfig import StorageConfig

class TestWikidata(Geograpy3Test):
    '''
    test the wikidata access for cities
    '''

    def testWikidataCountries(self):
        '''
        test getting country information from wikidata
        '''
        wikidata=Wikidata()
        try:
            countryList=wikidata.getCountries()
            self.assertTrue(len(countryList)>=200)
            expectedAttrs=Country.getSamples()[0].keys()
            for country in countryList:
                if self.debug:
                    print(country)
                    for attr in expectedAttrs:
                        self.assertTrue(hasattr(country,attr))
        except Exception as ex:
            self.handleWikidataException(ex)
            pass
        
    def testWikidataRegions(self):
        '''
        test getting region information from wikidata
        '''
        wikidata=Wikidata()
        try:
            regionList=wikidata.getRegions()
            self.assertTrue(len(regionList)>=3000)
        except Exception as ex:
            self.handleWikidataException(ex)
            pass

    def testWikidataCities(self):
        '''
        test getting city information from wikidata
        
        '''
        # Wikidata time outs in CI environment need to be avoided
        if getpass.getuser()!="wf":
            return
        config=StorageConfig.getSQL(debug=self.debug)
        config.cacheRootDir="/tmp/wdhs"
        cachedir=config.getCachePath()
        config.cacheFile=f"{cachedir}/hs.db"
        # use 2018 wikidata copy
        # wikidata.endpoint="http://blazegraph.bitplan.com/sparql"
        # use 2020 wikidata copy
        wikidata=Wikidata()
        wikidata.endpoint="https://confident.dbis.rwth-aachen.de/jena/wdhs/sparql"
        #wikidata.endpoint="http://jena.bitplan.com/wdhs/sparql"
        regions=[
            {"name": "Singapore", "country": "Q334", "region": None, "cities":46},
            {"name": "Beijing", "country": None, "region": "Q956", "cities":25},
            {"name": "Paris","country": None, "region": "Q13917", "cities":1242},
            {"name": "Barcelona","country": None, "region": "Q5705", "cities":1242},
            {"name": "Rome","country": None, "region": "Q1282", "cities":1242}
        ]
        limit=1000000 #if self.inCI() else 100
        cityList=wikidata.getCities(limit=limit)
        sqlDB=SQLDB(config.cacheFile)
        entityInfo=sqlDB.createTable(cityList,"hs",withDrop=True)
        sqlDB.store(cityList, entityInfo,fixNone=True)
        expected=200000 # if self.inCI() else limit
        self.assertTrue(len(cityList)>=expected)
        #for region in regions:
        #    starttime=time.time()
        #    regionName=region["name"]
        #    print(f"searching cities for {regionName}" )
        #    cityList=wikidata.getCities(country=region["country"], region=region["region"])
        #    print("Found %d cities for %s in %5.1f s" % (len(cityList),region["name"],time.time()-starttime))
        #    if self.debug:
        #        print(cityList[:10])
        #    #self.assertEqual(region['cities'],len(cityList))
        #    pass

    def testWikidataCityStates(self):
        '''
        test getting region information from wikidata
        '''
        wikidata=Wikidata()
        try:
            regionList=wikidata.getCityStates()
            self.assertTrue(len(regionList)>=2)
            cityStateNames=[r.get('name') for r in regionList]
            self.assertTrue("Singapore" in cityStateNames)
        except Exception as ex:
            self.handleWikidataException(ex)
            pass

    def testGetWikidataId(self):
        '''
        test getting a wikiDataId from a given URL
        '''
        # test entity
        wikidataURL="https://www.wikidata.org/wiki/Q1"
        expectedID="Q1"
        wikiDataId=Wikidata.getWikidataId(wikidataURL)
        self.assertEqual(wikiDataId, expectedID)
        # test property
        wikidataURLProperty="https://www.wikidata.org/wiki/Property:P31"
        expectedPropertyID="P31"
        propertyId=Wikidata.getWikidataId(wikidataURLProperty)
        self.assertEqual(expectedPropertyID, propertyId)
        # test invalid entries
        wikidataURLProperty = ""
        parsedId = Wikidata.getWikidataId(wikidataURLProperty)
        self.assertIsNone(parsedId)

    def testGetCoordinateComponents(self):
        '''
        test the splitting of coordinate components in WikiData query results
        '''
        cList=[
            {
              "coordinate":'Point(-118.25 35.05694444)',
              "expected":(-118.25,35.05694444)
            }
        ]
        for c  in cList:
            coordinate=c["coordinate"]
            expLat,expLon=c["expected"]
            lon, lat=Wikidata.getCoordinateComponents(coordinate)
            self.assertEqual(expLat, lat)
            self.assertEqual(expLon, lon)

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



'''
Created on 2021-08-16

@author: wf
'''
import unittest
from tests.basetest import Geograpy3Test
from geograpy.locator import City,CityManager,CountryManager,RegionManager,LocationContext
from geograpy.wikidata import Wikidata
from geograpy.utils import Profiler
import json
import os
import re
import getpass

class TestCachingCitiesByRegion(Geograpy3Test):
    '''
    The wikidata city query times out even on the wikidata copy in the RWTH i5 infrastructure
    Therefore we need to split the queries to a reasonable size so that each individual query does not time out.
    
    A query per region is done some 3000 times.
    The query used here works for most regions except a few where the query needs to be modified to not go the full depth of
    the Property
       located in the administrative territorial entity (P131) 
    but limit it
    
    '''

    def cacheRegionCities2Json(self,limit,showDone=False):
        # TODO - refactor to Locator/LocationContext - make available via command line
        wd=Wikidata()
        config=LocationContext.getDefaultConfig()
        countryManager= CountryManager(config=config)
        countryManager.fromCache()
        regionManager = RegionManager(config=config)
        regionManager.fromCache()
        regionList=regionManager.getList()   
        total=len(regionList) 
        cachePath=f"{config.getCachePath()}/regions"
        if not os.path.exists(cachePath):
                os.makedirs(cachePath)
        for index,region in enumerate(regionList):
            if index>=limit:
                break
            regionId=region.wikidataid
            msg=f"{index+1:4d}/{total:4d}:getting cities for {region.name} {region.iso} {region.wikidataid}"
            jsonFileName=f"{cachePath}/{region.iso}.json"
            if os.path.isfile(jsonFileName):
                if showDone:
                    print(msg)
            else:
                try:
                    regionCities=wd.getCitiesForRegion(regionId, msg)
                    jsonStr=json.dumps(regionCities)
                    with open(jsonFileName,"w") as jsonFile:
                        jsonFile.write(jsonStr)
                except Exception as ex:
                    self.handleWikidataException(ex)
                    

    def testGetCitiesByRegion(self):
        '''
        test counting human settlement types
        '''
        if self.inCI():
            limit=50
        elif getpass.getuser()=="wf":
            limit=5000
        else:
            limit=0
        self.cacheRegionCities2Json(limit=limit)
        
    def testReadCachedCitiesByRegion(self):
        '''
        test reading the cached json Files
        '''
        # This is to populate the cities database 
        return
        config=LocationContext.getDefaultConfig()
        regionManager = RegionManager(config=config)
        regionManager.fromCache()
        regionByIso,_dup=regionManager.getLookup("iso")
        self.assertEqual(56,len(_dup))
        jsonFiles=CityManager.getJsonFiles(config)
        msg=f"reading {len(jsonFiles)} cached city by region JSON cache files"
        self.assertTrue(len(jsonFiles)>2000)
        profiler=Profiler(msg)
        cityManager=CityManager(config=config)
        cityManager.getList().clear()
        for jsonFileName in jsonFiles:
            isoMatch = re.search(r"/([^\/]*)\.json", jsonFileName)
            if not isoMatch:
                print(f"{jsonFileName} - does not match a known region's ISO code")
            else:
                rIso=isoMatch.group(1)
                region=regionByIso[rIso]
                with open(jsonFileName) as jsonFile:
                    cities4Region = json.load(jsonFile)
                    for city4Region in cities4Region:
                        city=City()
                        city.fromDict(city4Region)
                        # fix regionId
                        if hasattr(city, "regionId"):
                            city.partOfRegionId=city.regionId
                        city.regionId=region.wikidataid
                        cityManager.add(city)
                        pass
        cityManager.store()
        profiler.time()

    def testCityFromCityStates(self):
        '''
        tests if city states are queried correctly if given the region
        For city states the city is region and city (in some cases also country).
        This test ensures that by querying for the cities of a region the city states include themself in the result
        (the result for cities in city-states often includes the municipalities)
        '''
        wd = Wikidata()
        cityStateRecords = wd.getCityStates()
        for cityStateRecord in cityStateRecords:
            regionId = cityStateRecord.get('wikidataid')
            regionCities = wd.getCitiesForRegion(regionId, msg=f"Query for cities in {cityStateRecord.get('name')}")
            foundCities=[c.get('wikidataid') for c in regionCities]
            self.assertTrue( regionId in foundCities)


if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()


'''
Created on 2021-08-17

@author: th
'''
import math
import unittest
from lodstorage.sql import SQLDB
from tests.basetest import Geograpy3Test
from geograpy.locator import RegionManager, LocationContext, CountryManager, CityManager
from geograpy.wikidata import Wikidata


class TestCachingLocationLabels(Geograpy3Test):
    '''
    adds location label tables
    
    '''

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testCacheLocationLabels(self):
        '''
        Generates the location label tabels in the SQL db fro countries, regions and cities by querying wikidata for
        the rdfs:label and skos:altLa of each location.
        A view containing all location labels is also created.
        '''
        testLocationLabelExtraction = False
        if testLocationLabelExtraction:
            wd=Wikidata()
            config = LocationContext.getDefaultConfig()
            countryManager=CountryManager(config=config)
            regionManager = RegionManager(config=config)
            cityManager=CityManager(config=config)
            sqlDb=SQLDB(dbname=config.cacheFile, debug=self.debug)
            for manager in countryManager, regionManager, cityManager:
                manager.fromCache()
                wikidataIdQuery = f"SELECT DISTINCT wikidataid FROM {manager.entityPluralName}"
                wikidataIdQueryRes = sqlDb.query(wikidataIdQuery)
                wikidataIds = [l['wikidataid'] for l in wikidataIdQueryRes]

                chunkSize=1000
                iterations = math.ceil(len(wikidataIds) / chunkSize)
                progress = 0
                res=[]
                for i in range(iterations):
                    workOnIds = wikidataIds[i * chunkSize:(i + 1) * chunkSize]
                    progress += len(workOnIds)
                    index = 0
                    values = ""
                    for location in workOnIds:
                        spacer = "  \n\t\t\t" if index % 10 == 0 else " "
                        values += f"{spacer}wd:{wd.getWikidataId(location)}"
                        index += 1
                    query = self.getLablesQuery(values)
                    res.extend(wd.query(f"Query {i}/{iterations} - Querying {manager.entityName} Labels", queryString=query))
                wd.store2DB(res, tableName=f"{manager.entityName}_labels", sqlDB=sqlDb)
            self.createViews(sqlDB=sqlDb)


    def getLablesQuery(self, wikidataIds:str):
        '''
        get the query for the alternatives labels for the given values
        
        wikidataIds(str): a list of wikidataids
        '''
        query = """# get alternative labels for the given wikidata 
PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>
PREFIX skos: <http://www.w3.org/2004/02/skos/core#>
PREFIX wd: <http://www.wikidata.org/entity/>
SELECT DISTINCT ?wikidataid ?label ?lang
WHERE{
  VALUES ?wikidataid { %s }
  ?wikidataid rdfs:label|skos:altLabel ?label
  BIND(lang(?label) AS ?lang)
  FILTER(lang(?label)="en")
}""" % wikidataIds
        return query

    def createViews(self, sqlDB):
        viewDDLs = ["DROP VIEW IF EXISTS location_labels", """
                CREATE VIEW location_labels AS 
                SELECT *, "Country" AS "hierarchy" 
                FROM country_labels 
                UNION 
                SELECT *, "Region" AS "hierarchy" 
                FROM region_labels 
                UNION 
                SELECT *, "City" AS "hierarchy" 
                FROM city_labels
        """,
                    "DROP INDEX if EXISTS cityLabelByWikidataid",
                    "CREATE INDEX cityLabelByWikidataid ON city_labels (wikidataid)",
                    "DROP INDEX if EXISTS regionLabelByWikidataid",
                    "CREATE INDEX regionLabelByWikidataid ON region_labels (wikidataid)",
                    "DROP INDEX if EXISTS countryLabelByWikidataid",
                    "CREATE INDEX countryLabelByWikidataid ON country_labels (wikidataid)",]
        for viewDDL in viewDDLs:
            sqlDB.execute(viewDDL)


if __name__ == "__main__":
    # import sys;sys.argv = ['', 'Test.testName']
    unittest.main()


'''
Created on 2021-06-09

@author: wf
'''
import unittest
import numpy as np
from geograpy.locator import Locator, LocationManager, CityManager, CountryManager, RegionManager, Country, LocationContext
from sklearn.neighbors import BallTree

from math import radians
from tests.basetest import Geograpy3Test

class TestLocationHierarchy(Geograpy3Test):
    '''
    tests for the location hierarchy
    '''

    def setUp(self):
        super().setUp()
        self.locationContext = None
        pass

    def getLocationContext(self):
        if self.locationContext is None:
            self.locationContext = LocationContext.fromCache()
        return self.locationContext
            
    def testDistance(self):
        '''
        test calculcating the distance of two points using the haversine function
        '''
        # https://stackoverflow.com/a/64585765/1497139
        earth_radius = 6371000  # meters in earth
        test_radius = 1300000  # meters
        
        test_points = [[32.027240, -81.093190], [41.981876, -87.969982]]
        test_points_rad = np.array([[radians(x[0]), radians(x[1])] for x in test_points ])
        
        tree = BallTree(test_points_rad, metric='haversine')
        ind, results = tree.query_radius(test_points_rad, r=test_radius / earth_radius,
        return_distance=True)
        if self.debug:
            print(ind)
            print(results * earth_radius / 1000)

    def testIssue45_BallTree(self):
        '''
        test calculation a ball tree for a given list of locations
        '''
        countryList = CountryManager.fromErdem()
        ballTree, validList = countryList.getBallTuple()
        self.assertEqual(245, len(validList))
        self.assertEqual("BallTree", type(ballTree).__name__)
        self.assertAlmostEqual(245, ballTree.sum_weight, delta=0.1)
        pass

    def checkLocationListWithDistances(self, locationListWithDistances, expectedCount, expectedClosest, expectedDistance):
        '''
        check the location list with the given distances
        '''
        if self.debug:
            for i, locationWithDistance in enumerate(locationListWithDistances):
                location, distance = locationWithDistance
                print(f"{i}:{location}-{distance:.0f} km")
        self.assertEqual(len(locationListWithDistances), expectedCount)
        closestLocation, distance = locationListWithDistances[0]
        self.assertEqual(expectedClosest, closestLocation.name)
        self.assertAlmostEqual(expectedDistance, distance, delta=1)

    def testClosestLocation(self):
        '''
        test getting the closes Location to a given location
        '''
        # sample Country: Germany
        country = Country()
        country.name = 'Germany'
        country.lat = 51.0
        country.lon = 9.0
        # get a country list
        lookupCountryManager = CountryManager.fromErdem()
        # get the closest 2 locations for the given countryList
        countryListWithDistances = country.getNClosestLocations(lookupCountryManager, 2)
        self.checkLocationListWithDistances(countryListWithDistances, 2, "Luxembourg", 244)

        countryListWithDistances = country.getLocationsWithinRadius(lookupCountryManager, 300)
        self.checkLocationListWithDistances(countryListWithDistances, 2, "Luxembourg", 244)

    def testRegionMatching(self):
        '''
        test region matches
        '''
        locator = Locator()
        if not locator.db_has_data():
            locator.populate_db()
        countryList = CountryManager.fromErdem()
        config=LocationContext.getDefaultConfig()
        regionManager=RegionManager(config=config)
        regionManager.fromCache()
        for country in countryList.countries:
            locationListWithDistances = country.getNClosestLocations(regionManager, 3)
            if self.debug:
                print(f"{country}{country.lat:.2f},{country.lon:.2f}")
            for i, locationWithDistance in enumerate(locationListWithDistances):
                location, distance = locationWithDistance
                if self.debug:
                    print(f"    {i}:{location}-{distance:.0f} km")
        pass

    def testLocationListLoading(self):
        '''
        test loading the locations from Json
        '''
        samples = """
        {
            "countries": [
                {
                    "name": "Afghanistan",
                    "wikidataid": "Q889",
                    "lat": 34,
                    "lon": 66,
                    "coordinates": "34,66",
                    "partOf": null,
                    "level": 3,
                    "locationKind": "Country",
                    "comment": null,
                    "iso": "AF"
                },
                {
                    "name": "United States of America",
                    "wikidataid": "Q30",
                    "lat": 39.82818,
                    "lon": -98.5795,
                    "partOf": "Noth America",
                    "level": 3,
                    "locationKind": "Country",
                    "comment": null,
                    "labels": [
                        "America",
                        "UNITED STATES OF AMERICA",
                        "USA",
                        "United States",
                        "United States of America (the)"
                    ],
                    "iso": "US"
                },
                {
                    "name": "Australia",
                    "wikidataid": "Q408",
                    "lat": -28,
                    "lon": 137,
                    "coordinates": "-28,137",
                    "partOf": null,
                    "level": 3,
                    "locationKind": "Country",
                    "comment": null,
                    "labels": [
                        "AUS"
                    ],
                    "iso": "AU"
                }
            ]
        }
        """
        cm = CountryManager();
        cm.restoreFromJsonStr(samples)
        countriesByWikiDataId, _dup = cm.getLookup("wikidataid")
        self.assertTrue("Q30" in countriesByWikiDataId)
    

    def test_getLocationByID(self):
        '''
        tests if the correct location for a given wikidataid is returned
        '''
        config=LocationContext.getDefaultConfig()
        countryManager = CountryManager(config=config)
        countryManager.fromCache()
        country = countryManager.getLocationByID("Q30")  # wikidataid of USA
        self.assertIsNotNone(country)
        self.assertTrue(hasattr(country, 'iso'))
        self.assertEqual(country.iso, 'US')

    def test_LocationContext(self):
        '''
        tests the LocationContext class
        '''

        # test interlinking of city with region and country
        locationContext = self.getLocationContext()
        cities = locationContext.cityManager.getByName('Los Angeles')
        la = [x for x in cities if x.wikidataid == "Q65"][0]
        self.assertEqual(la.name, 'Los Angeles')
        ca = la.region
        self.assertEqual(ca.name, 'California')
        us = la.country
        self.assertEqual(us.wikidataid, 'Q30')
        self.assertEqual(la.country, ca.country)

    def testLocateLocation(self):
        '''
        test LocationContext locateLocation
        '''
        exampleLocations = {
            "Washington, DC, USA": "Q61",
            "Bangalore": "Q1355",
            "Bangalore, India": "Q1355",
            "Xi'an": "Q5826",
            "Xi'an, China": "Q5826",
            "Virtual Event USA": "Q30",
            "Virtual USA": "Q30",
            "London United Kingdom": "Q84",
            "Brno":"Q14960",
            "Cancun":"Q8969",
            "St. Petersburg": "Q656",
            "Gothenburg Sweden": "Q25287",
            "Los Angeles California": "Q65",
            "Zurich, Switzerland": "Q72",
            "Barcelona Spain": "Q1492",
            "Vienna Austria": "Q1741",
            "Seoul Republic of Korea": "Q8684",
            "Seattle WA USA": "Q5083",
            "Singapore Singapore":"Q334",
            "Tokyo Japan": "Q1490",
            "Vancouver BC Canada": "Q24639",
            "Vancouver British Columbia Canada": "Q24639",
            "Amsterdam Netherlands":"Q727",
            "Paris France": "Q90",
            "Nagoya": "Q11751",
            "Marrakech":"Q101625",
            "Austin Texas":"Q16559",
            "Chicago IL USA":"Q1297",
            "Bangkok Thailand":"Q1861",
            "Firenze, Italy":"Q2044",
            "Florence Italy":"Q2044",
            "Timisoara":"Q83404",
            "Langkawi":"Q273303",
            "Beijing China":"Q956",
            "Berlin Germany": "Q64",
            "Prague Czech Republic":"Q1085",
            "Portland Oregon USA":"Q6106",
            "Portland OR USA":"Q6106",
            "Pittsburgh PA USA":"Q1342",
            "Новосибирск":"Q883",
            "Los Angeles CA USA":"Q65",
            "Kyoto Japan": "Q34600"
        }
        locationContext = self.getLocationContext()
        printPretty = lambda records:print([str(record) for record in records])
        failures = []
        for locationText in exampleLocations.keys():
            expectedLocationId = exampleLocations[locationText] 
            locations = locationContext.locateLocation(locationText, verbose=True)
            if len(locations) < 1:
                failures.append(locationText)
            else:
                location = locations[0]
                if self.debug:
                    printPretty(location)
                if not location.wikidataid == expectedLocationId:
                    failures.append(locationText)
        showFailures=True
        if self.debug or showFailures:
            print(f"locationLooup failed for {failures}")
        self.assertTrue(len(failures)<=40)
        
    def testLocateLocationCountryRegionCity(self):
        '''
        test LocationContext locateLocation
        '''
        locationContext = self.getLocationContext()
        printPretty = lambda records:print([str(record) for record in records])
 
        pl1 = locationContext.locateLocation("Berlin", "USA")
        self.assertEqual("Germany", pl1[0].country.name)
        if self.debug:
            printPretty(pl1)
        pl2 = locationContext.locateLocation("Los Angeles, CA")
        if self.debug:
            printPretty(pl2)
        self.assertEqual("California", pl2[0].region.name)
        pl3 = locationContext.locateLocation("Germany, Aachen")
        if self.debug:
            printPretty(pl3)
        self.assertEqual("Aachen", pl3[0].name)
        self.assertEqual("Germany", pl3[0].country.name)


if __name__ == "__main__":
    # import sys;sys.argv = ['', 'Test.testName']
    unittest.main()
